package sun.font;

import java.nio.IntBuffer;
import java.nio.CharBuffer;
import java.nio.charset.CodingErrorAction;
import java.nio.charset.Charset;
import java.nio.ByteBuffer;

abstract class CMap
{
    static final short ShiftJISEncoding = 2;
    static final short GBKEncoding = 3;
    static final short Big5Encoding = 4;
    static final short WansungEncoding = 5;
    static final short JohabEncoding = 6;
    static final short MSUnicodeSurrogateEncoding = 10;
    static final char noSuchChar = '�';
    static final int SHORTMASK = 65535;
    static final int INTMASK = Integer.MAX_VALUE;
    static final char[][] converterMaps;
    char[] xlat;
    public static final CMap.NullCMapClass theNullCmap;
    
    CMap() {
    }
    
    static CMap initialize(TrueTypeFont trueTypeFont) {
        CMap cMap = null;
        int n = 0;
        int n2 = 0;
        int n3 = 0;
        int n4 = 0;
        int n5 = 0;
        int n6 = 0;
        int n7 = 0;
        int n8 = 0;
        boolean n9 = false;
        ByteBuffer tableBuffer = trueTypeFont.getTableBuffer(1668112752);
        trueTypeFont.getTableSize(1668112752);
        for (int short1 = tableBuffer.getShort(2), i = 0; i < short1; ++i) {
            tableBuffer.position(i * 8 + 4);
            if (tableBuffer.getShort() == 3) {
                n9 = true;
                int short2 = tableBuffer.getShort();
                int int1 = tableBuffer.getInt();
                switch (short2) {
                    case 0: {
                        n = int1;
                        break;
                    }
                    case 1: {
                        n2 = int1;
                        break;
                    }
                    case 2: {
                        n3 = int1;
                        break;
                    }
                    case 3: {
                        n4 = int1;
                        break;
                    }
                    case 4: {
                        n5 = int1;
                        break;
                    }
                    case 5: {
                        n6 = int1;
                        break;
                    }
                    case 6: {
                        n7 = int1;
                        break;
                    }
                    case 10: {
                        n8 = int1;
                        break;
                    }
                }
            }
        }
        if (!(n9 == 0)) {
            if (n8 != 0) {
                cMap = createCMap(tableBuffer, n8, (char[])null);
            }
            else if (n != 0) {
                cMap = createCMap(tableBuffer, n, (char[])null);
            }
            else if (n2 != 0) {
                cMap = createCMap(tableBuffer, n2, (char[])null);
            }
            else if (n3 != 0) {
                cMap = createCMap(tableBuffer, n3, getConverterMap((short)2));
            }
            else if (n4 != 0) {
                cMap = createCMap(tableBuffer, n4, getConverterMap((short)3));
            }
            else if (n5 != 0) {
                if (FontUtilities.isSolaris && trueTypeFont.platName != null && (trueTypeFont.platName.startsWith("/usr/openwin/lib/locale/zh_CN.EUC/X11/fonts/TrueType") || trueTypeFont.platName.startsWith("/usr/openwin/lib/locale/zh_CN/X11/fonts/TrueType") || trueTypeFont.platName.startsWith("/usr/openwin/lib/locale/zh/X11/fonts/TrueType"))) {
                    cMap = createCMap(tableBuffer, n5, getConverterMap((short)3));
                }
                else {
                    cMap = createCMap(tableBuffer, n5, getConverterMap((short)4));
                }
            }
            else if (n6 != 0) {
                cMap = createCMap(tableBuffer, n6, getConverterMap((short)5));
            }
            else if (n7 != 0) {
                cMap = createCMap(tableBuffer, n7, getConverterMap((short)6));
            }
        }
        else {
            cMap = createCMap(tableBuffer, tableBuffer.getInt(8), (char[])null);
        }
        return cMap;
    }
    
    static char[] getConverter(short n) {
        int i;
        int n2;
        String charsetName;
        switch (n) {
            case 2: {
                i = 33088;
                n2 = 64764;
                charsetName = "SJIS";
                break;
            }
            case 3: {
                i = 33088;
                n2 = 65184;
                charsetName = "GBK";
                break;
            }
            case 4: {
                i = 41280;
                n2 = 65278;
                charsetName = "Big5";
                break;
            }
            case 5: {
                i = 41377;
                n2 = 65246;
                charsetName = "EUC_KR";
                break;
            }
            case 6: {
                i = 33089;
                n2 = 65022;
                charsetName = "Johab";
                break;
            }
            default: {
                return null;
            }
        }
        try {
            char[] array = new char[65536];
            for (int j = 0; j < 65536; ++j) {
                array[j] = '�';
            }
            byte[] array2 = new byte[(n2 - i + 1) * 2];
            char[] array3 = new char[n2 - i + 1];
            int n3 = 0;
            if (n == 2) {
                for (; i <= n2; ++i) {
                    int n4 = i >> 8 & 0xFF;
                    if (n4 >= 161 && n4 <= 223) {
                        array2[n3++] = -1;
                        array2[n3++] = -1;
                    }
                    else {
                        array2[n3++] = (byte)n4;
                        array2[n3++] = (byte)(i & 0xFF);
                    }
                }
            }
            else {
                for (; i <= n2; ++i) {
                    array2[n3++] = (byte)(i >> 8 & 0xFF);
                    array2[n3++] = (byte)(i & 0xFF);
                }
            }
            Charset.forName(charsetName).newDecoder().onMalformedInput(CodingErrorAction.REPLACE).onUnmappableCharacter(CodingErrorAction.REPLACE).replaceWith("\u0000").decode(ByteBuffer.wrap(array2, 0, array2.length), CharBuffer.wrap(array3, 0, array3.length), true);
            for (int k = 32; k <= 126; ++k) {
                array[k] = (char)k;
            }
            if (n == 2) {
                for (int l = 161; l <= 223; ++l) {
                    array[l] = (char)(l - 161 + 65377);
                }
            }
            System.arraycopy(array3, 0, array, i, array3.length);
            char[] array4 = new char[65536];
            for (int n5 = 0; n5 < 65536; ++n5) {
                if (array[n5] != '�') {
                    array4[array[n5]] = (char)n5;
                }
            }
            return array4;
        }
        catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }
    
    static char[] getConverterMap(short n) {
        if (CMap.converterMaps[n] == null) {
            CMap.converterMaps[n] = getConverter(n);
        }
        return CMap.converterMaps[n];
    }
    
    static CMap createCMap(ByteBuffer byteBuffer, int n, char[] array) {
        int char1 = byteBuffer.getChar(n);
        long n2;
        if (char1 < '\b') {
            n2 = (long)byteBuffer.getChar(n + 2);
        }
        else {
            n2 = (long)(byteBuffer.getInt(n + 4) & Integer.MAX_VALUE);
        }
        if ((long)n + n2 > (long)byteBuffer.capacity() && FontUtilities.isLogging()) {
            FontUtilities.getLogger().warning("Cmap subtable overflows buffer.");
        }
        switch (char1) {
            case 0: {
                return new CMap.CMapFormat0(byteBuffer, n);
            }
            case 2: {
                return new CMap.CMapFormat2(byteBuffer, n, array);
            }
            case 4: {
                return new CMap.CMapFormat4(byteBuffer, n, array);
            }
            case 6: {
                return new CMap.CMapFormat6(byteBuffer, n, array);
            }
            case 8: {
                return new CMap.CMapFormat8(byteBuffer, n, array);
            }
            case 10: {
                return new CMap.CMapFormat10(byteBuffer, n, array);
            }
            case 12: {
                return new CMap.CMapFormat12(byteBuffer, n, array);
            }
            default: {
                throw new RuntimeException("Cmap format unimplemented: " + (int)byteBuffer.getChar(n));
            }
        }
    }
    
    abstract char getGlyph(int p0);
    
    final int getControlCodeGlyph(int n, boolean b) {
        if (n < 16) {
            switch (n) {
                case 9:
                case 10:
                case 13: {
                    return 65535;
                }
                default: {
                    break;
                }
            }
        }
        else if (n >= 8204) {
            if (n <= 8207 || (n >= 8232 && n <= 8238) || (n >= 8298 && n <= 8303)) {
                return 65535;
            }
            if (b && n >= 65535) {
                return 0;
            }
        }
        return -1;
    }
    
    static {
        converterMaps = new char[7][];
        theNullCmap = new CMap.NullCMapClass();
    }
    
    static class CMapFormat4 extends CMap
    {
        int segCount;
        int entrySelector;
        int rangeShift;
        char[] endCount;
        char[] startCount;
        short[] idDelta;
        char[] idRangeOffset;
        char[] glyphIds;
        
        CMapFormat4(ByteBuffer byteBuffer, int n, char[] xlat) {
            this.xlat = xlat;
            byteBuffer.position(n);
            CharBuffer charBuffer = byteBuffer.asCharBuffer();
            charBuffer.get();
            int value = charBuffer.get();
            if (n + value > byteBuffer.capacity()) {
                value = byteBuffer.capacity() - n;
            }
            charBuffer.get();
            this.segCount = charBuffer.get() / 2;
            charBuffer.get();
            this.entrySelector = charBuffer.get();
            this.rangeShift = charBuffer.get() / 2;
            this.startCount = new char[this.segCount];
            this.endCount = new char[this.segCount];
            this.idDelta = new short[this.segCount];
            this.idRangeOffset = new char[this.segCount];
            for (int i = 0; i < this.segCount; ++i) {
                this.endCount[i] = charBuffer.get();
            }
            charBuffer.get();
            for (int j = 0; j < this.segCount; ++j) {
                this.startCount[j] = charBuffer.get();
            }
            for (int k = 0; k < this.segCount; ++k) {
                this.idDelta[k] = (short)charBuffer.get();
            }
            for (int l = 0; l < this.segCount; ++l) {
                this.idRangeOffset[l] = (char)(charBuffer.get() >> 1 & 0xFFFF);
            }
            int n2 = (this.segCount * 8 + 16) / 2;
            charBuffer.position(n2);
            int n3 = value / 2 - n2;
            this.glyphIds = new char[n3];
            for (int n4 = 0; n4 < n3; ++n4) {
                this.glyphIds[n4] = charBuffer.get();
            }
        }
        
        @Override
        char getGlyph(int n) {
            int n2 = 0;
            int controlCodeGlyph = this.getControlCodeGlyph(n, true);
            if (controlCodeGlyph >= 0) {
                return (char)controlCodeGlyph;
            }
            if (this.xlat != null) {
                n = this.xlat[n];
            }
            int i = 0;
            int length = this.startCount.length;
            int n3 = this.startCount.length >> 1;
            while (i < length) {
                if (this.endCount[n3] < n) {
                    i = n3 + 1;
                }
                else {
                    length = n3;
                }
                n3 = i + length >> 1;
            }
            if (n >= this.startCount[n3] && n <= this.endCount[n3]) {
                int n4 = this.idRangeOffset[n3];
                if (n4 == '\0') {
                    n2 = (char)(n + this.idDelta[n3]);
                }
                else {
                    n2 = this.glyphIds[n4 - this.segCount + n3 + (n - this.startCount[n3])];
                    if (n2 != 0) {
                        n2 = (char)(n2 + this.idDelta[n3]);
                    }
                }
            }
            return n2;
        }
    }
    
    static class CMapFormat0 extends CMap
    {
        byte[] cmap;
        
        CMapFormat0(ByteBuffer byteBuffer, int n) {
            this.cmap = new byte[byteBuffer.getChar(n + 2) - 6];
            byteBuffer.position(n + 6);
            byteBuffer.get(this.cmap);
        }
        
        @Override
        char getGlyph(int n) {
            if (n < 256) {
                if (n < 16) {
                    switch (n) {
                        case 9:
                        case 10:
                        case 13: {
                            return '\uffff';
                        }
                        default: {
                            break;
                        }
                    }
                }
                return (char)(0xFF & this.cmap[n]);
            }
            else {
                return '\0';
            }
        }
    }
    
    static class CMapFormat2 extends CMap
    {
        char[] subHeaderKey;
        char[] firstCodeArray;
        char[] entryCountArray;
        short[] idDeltaArray;
        char[] idRangeOffSetArray;
        char[] glyphIndexArray;
        
        CMapFormat2(ByteBuffer byteBuffer, int n, char[] xlat) {
            this.subHeaderKey = new char[256];
            this.xlat = xlat;
            int char1 = byteBuffer.getChar(n + 2);
            byteBuffer.position(n + 6);
            CharBuffer charBuffer = byteBuffer.asCharBuffer();
            int n2 = '\0';
            for (int i = 0; i < 256; ++i) {
                this.subHeaderKey[i] = charBuffer.get();
                if (this.subHeaderKey[i] > n2) {
                    n2 = this.subHeaderKey[i];
                }
            }
            int n3 = (n2 >> 3) + 1;
            this.firstCodeArray = new char[n3];
            this.entryCountArray = new char[n3];
            this.idDeltaArray = new short[n3];
            this.idRangeOffSetArray = new char[n3];
            for (int j = 0; j < n3; ++j) {
                this.firstCodeArray[j] = charBuffer.get();
                this.entryCountArray[j] = charBuffer.get();
                this.idDeltaArray[j] = (short)charBuffer.get();
                this.idRangeOffSetArray[j] = charBuffer.get();
            }
            int n4 = (char1 - 518 - n3 * 8) / 2;
            this.glyphIndexArray = new char[n4];
            for (int k = 0; k < n4; ++k) {
                this.glyphIndexArray[k] = charBuffer.get();
            }
        }
        
        @Override
        char getGlyph(int n) {
            int controlCodeGlyph = this.getControlCodeGlyph(n, true);
            if (controlCodeGlyph >= 0) {
                return (char)controlCodeGlyph;
            }
            if (this.xlat != null) {
                n = this.xlat[n];
            }
            int n2 = (char)(n >> 8);
            int n3 = (char)(n & 0xFF);
            int n4 = this.subHeaderKey[n2] >> 3;
            if (n4 != 0) {
            }
            else {
                n3 = n2;
                if (n3 == '\0') {
                }
            }
            int n5 = this.firstCodeArray[n4];
            if (n3 < n5) {
                return '\0';
            }
            int n6 = (char)(n3 - n5);
            if (n6 < this.entryCountArray[n4]) {
                int n7 = this.glyphIndexArray[(this.idRangeOffSetArray[n4] - ((this.idRangeOffSetArray.length - n4) * 8 - 6)) / 2 + n6];
                if (n7 != '\0') {
                    return (char)(n7 + this.idDeltaArray[n4]);
                }
            }
            return '\0';
        }
    }
    
    static class CMapFormat6 extends CMap
    {
        char firstCode;
        char entryCount;
        char[] glyphIdArray;
        
        CMapFormat6(ByteBuffer byteBuffer, int n, char[] array) {
            byteBuffer.position(n + 6);
            CharBuffer charBuffer = byteBuffer.asCharBuffer();
            this.firstCode = charBuffer.get();
            this.entryCount = charBuffer.get();
            this.glyphIdArray = new char[this.entryCount];
            for (int i = '\0'; i < this.entryCount; ++i) {
                this.glyphIdArray[i] = charBuffer.get();
            }
        }
        
        @Override
        char getGlyph(int n) {
            int controlCodeGlyph = this.getControlCodeGlyph(n, true);
            if (controlCodeGlyph >= 0) {
                return (char)controlCodeGlyph;
            }
            if (this.xlat != null) {
                n = this.xlat[n];
            }
            n -= this.firstCode;
            if (n < 0 || n >= this.entryCount) {
                return '\0';
            }
            return this.glyphIdArray[n];
        }
    }
    
    static class CMapFormat8 extends CMap
    {
        byte[] is32;
        int nGroups;
        int[] startCharCode;
        int[] endCharCode;
        int[] startGlyphID;
        
        CMapFormat8(ByteBuffer byteBuffer, int n, char[] array) {
            this.is32 = new byte[8192];
            byteBuffer.position(12);
            byteBuffer.get(this.is32);
            this.nGroups = (byteBuffer.getInt() & Integer.MAX_VALUE);
            if ((long)byteBuffer.remaining() < 12L * (long)this.nGroups) {
                throw new RuntimeException("Format 8 table exceeded");
            }
            this.startCharCode = new int[this.nGroups];
            this.endCharCode = new int[this.nGroups];
            this.startGlyphID = new int[this.nGroups];
            return;
        }
        
        @Override
        char getGlyph(int n) {
            if (this.xlat != null) {
                throw new RuntimeException("xlat array for cmap fmt=8");
            }
            return '\0';
        }
    }
    
    static class CMapFormat10 extends CMap
    {
        long firstCode;
        int entryCount;
        char[] glyphIdArray;
        
        CMapFormat10(ByteBuffer byteBuffer, int n, char[] array) {
            byteBuffer.position(n + 12);
            this.firstCode = (long)(byteBuffer.getInt() & Integer.MAX_VALUE);
            this.entryCount = (byteBuffer.getInt() & Integer.MAX_VALUE);
            if ((long)byteBuffer.remaining() < 2L * (long)this.entryCount) {
                throw new RuntimeException("Format 10 table exceeded");
            }
            CharBuffer charBuffer = byteBuffer.asCharBuffer();
            this.glyphIdArray = new char[this.entryCount];
            for (int i = 0; i < this.entryCount; ++i) {
                this.glyphIdArray[i] = charBuffer.get();
            }
            return;
        }
        
        @Override
        char getGlyph(int n) {
            if (this.xlat != null) {
                throw new RuntimeException("xlat array for cmap fmt=10");
            }
            int n2 = (int)((long)n - this.firstCode);
            if (n2 < 0 || n2 >= this.entryCount) {
                return '\0';
            }
            return this.glyphIdArray[n2];
        }
    }
    
    static class CMapFormat12 extends CMap
    {
        int numGroups;
        int highBit;
        int power;
        int extra;
        long[] startCharCode;
        long[] endCharCode;
        int[] startGlyphID;
        
        CMapFormat12(ByteBuffer slice, int n, char[] array) {
            this.highBit = 0;
            if (array != null) {
                throw new RuntimeException("xlat array for cmap fmt=12");
            }
            slice.position(n + 12);
            this.numGroups = (slice.getInt() & Integer.MAX_VALUE);
            if ((long)slice.remaining() < 12L * (long)this.numGroups) {
                throw new RuntimeException("Format 12 table exceeded");
            }
            this.startCharCode = new long[this.numGroups];
            this.endCharCode = new long[this.numGroups];
            this.startGlyphID = new int[this.numGroups];
            slice = slice.slice();
            IntBuffer intBuffer = slice.asIntBuffer();
            for (int i = 0; i < this.numGroups; ++i) {
                this.startCharCode[i] = (long)(intBuffer.get() & Integer.MAX_VALUE);
                this.endCharCode[i] = (long)(intBuffer.get() & Integer.MAX_VALUE);
                this.startGlyphID[i] = (intBuffer.get() & Integer.MAX_VALUE);
            }
            int numGroups = this.numGroups;
            if (numGroups >= 65536) {
                numGroups >>= 16;
                this.highBit += 16;
            }
            if (numGroups >= 256) {
                numGroups >>= 8;
                this.highBit += 8;
            }
            if (numGroups >= 16) {
                numGroups >>= 4;
                this.highBit += 4;
            }
            if (numGroups >= 4) {
                numGroups >>= 2;
                this.highBit += 2;
            }
            if (numGroups >= 2) {
                ++this.highBit;
            }
            this.power = 1 << this.highBit;
            this.extra = this.numGroups - this.power;
            return;
        }
        
        @Override
        char getGlyph(int n) {
            int controlCodeGlyph = this.getControlCodeGlyph(n, false);
            if (controlCodeGlyph >= 0) {
                return (char)controlCodeGlyph;
            }
            int i = this.power;
            int extra = 0;
            if (this.startCharCode[this.extra] <= (long)n) {
                extra = this.extra;
            }
            while (i > 1) {
                i >>= 1;
                if (this.startCharCode[extra + i] > (long)n) {
                    continue;
                }
                extra += i;
            }
            if (this.startCharCode[extra] <= (long)n && this.endCharCode[extra] >= (long)n) {
                return (char)(int)((long)this.startGlyphID[extra] + ((long)n - this.startCharCode[extra]));
            }
            return '\0';
        }
    }
    
    static class NullCMapClass extends CMap
    {
        @Override
        char getGlyph(int n) {
            return '\0';
        }
    }
}
